import { makeError, VElement, VTextNode } from '@textbus/core';
import { ref, h } from 'vue';
import { DomAdapter } from '@textbus/platform-browser';

const adapterError = makeError('VueAdapter');
class VueAdapter extends DomAdapter {
    constructor(components, mount) {
        super({
            createCompositionNode: (compositionState, updateNativeCompositionNode) => {
                return new VElement('span', {
                    style: {
                        textDecoration: 'underline'
                    },
                    ref: updateNativeCompositionNode
                }, [
                    new VTextNode(compositionState.text)
                ]);
            },
            getParentNode(node) {
                return node.parentNode;
            },
            getChildNodes(parentElement) {
                return Array.from(parentElement.childNodes);
            },
            isNativeElementNode(node) {
                return node instanceof HTMLElement;
            },
            getChildByIndex(parentElement, index) {
                return parentElement.childNodes[index];
            },
            getAndUpdateSlotRootNativeElement(vElement, update) {
                const currentRef = vElement.attrs.get('ref');
                if (currentRef) {
                    vElement.attrs.set('ref', (v) => {
                        update(v);
                        if (typeof currentRef === 'function') {
                            currentRef(v);
                        }
                        else if (!currentRef.value) {
                            currentRef.value = v;
                        }
                    });
                }
                else {
                    vElement.attrs.set('ref', update);
                }
            },
            componentRender: (component) => {
                const comp = this.components[component.name] || this.components['*'];
                if (comp) {
                    let rootRef = this.componentRefs.get(component);
                    if (!rootRef) {
                        rootRef = ref();
                        this.componentRefs.set(component, rootRef);
                    }
                    return h(comp, {
                        component,
                        rootRef,
                        key: component.id
                    });
                }
                throw adapterError(`cannot found view component \`${component.name}\`!`);
            },
            vElementToViewElement(vNode, children) {
                const props = Object.assign({}, (Array.from(vNode.attrs).reduce((a, b) => {
                    a[b[0]] = b[1];
                    return a;
                }, {})));
                if (vNode.classes.size) {
                    props.class = Array.from(vNode.classes).join(' ');
                }
                if (vNode.styles) {
                    props.style = Array.from(vNode.styles).reduce((a, b) => {
                        a[b[0]] = b[1];
                        return a;
                    }, {});
                }
                return h(vNode.tagName, props, ...children);
            }
        }, mount);
        // private compositionRef = ref<HTMLElement>()
        this.componentRefs = new WeakMap();
        this.components = {};
    }
    copy() {
        document.execCommand('copy');
    }
}

export { VueAdapter };
